@@ -36,10 +36,12 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "nObserver.h"
 //#include "tCrypt.h"
 #include "tException.h"
+#include <memory>
 
 class nSocket;
 class nAddress;
 class nBasicNetworkSystem;
+class nServerInfoBase;
 
 extern nBasicNetworkSystem sn_BasicNetworkSystem;
 
@@ -62,15 +64,16 @@ extern const unsigned int sn_defaultPort; // default port a server listens on
 
 extern int sn_defaultDelay;
 
+extern  bool sn_decorateTS;
+
 extern tString sn_DenyReason;		// the reason the server gave for sending a login_deny packet
 
 // rate control
 extern int sn_maxRateIn,sn_maxRateOut;
 
-// exception that is thrown on any unexpected network error;
-// causes the owner of the current nNetObject or the sender of
-// the currently processed netmessage to be killed.
-
+//! exception that is thrown on any unexpected network error;
+//! causes the owner of the current nNetObject or the sender of
+//! the currently processed netmessage to be killed.
 class nKillHim: public tException
 {
 public:
@@ -82,24 +85,41 @@ private:
     virtual tString DoGetDescription()  const;              //!< returns a detailed description
 };
 
+//! exception that is thrown on errors where it is safe to just ignore the
+//! offending packet. 
+class nIgnore: public nKillHim
+{
+public:
+    nIgnore();
+    ~nIgnore();
+
+private:
+    virtual tString DoGetName()         const;              //!< returns the name of the exception
+    virtual tString DoGetDescription()  const;              //!< returns a detailed description
+};
+
 // call this function on any error occuring while reading a message:
-void nReadError();
+void nReadError( bool critical = true );
 
 #ifndef MAXCLIENTS
+#ifdef DEDICATED
+#define MAXCLIENTS 32
+#else
 #define MAXCLIENTS 16
 #endif
+#endif
 
 // We can be single player, multiplayer server/client.
 typedef enum {nSTANDALONE,nSERVER,nCLIENT} nNetState;
-typedef enum {nOK, nTIMEOUT, nDENIED}      nConnectError;
+typedef enum {nOK, nTIMEOUT, nDENIED, nABORT}   nConnectError;
 
 
 // set/get the state
 nConnectError sn_GetLastError();
 nNetState sn_GetNetState();
 void sn_SetNetState(nNetState x);
-void sn_DisconnectUser(int i, const tOutput& reason ); //!< terminate connection with user i (peacefully)
-void sn_KickUser(int i, const tOutput& reason, REAL severity = 1 );   //!< throw out user i (violently)
+void sn_DisconnectUser(int i, const tOutput& reason, nServerInfoBase * redirectTo = 0 ); //!< terminate connection with user i (peacefully)
+void sn_KickUser(int i, const tOutput& reason, REAL severity = 1, nServerInfoBase * redirectTo = 0 );   //!< throw out user i (violently)
 
 void sn_GetAdr(int user,  tString& name);
 unsigned int sn_GetPort(int user);
@@ -143,8 +163,8 @@ class nVersionFeature
 {
 public:
     nVersionFeature( int min, int max = -1 ); // creates a feature that is supported from version min to max; values of -1 indicate no border
-    bool Supported();                         // returns whether this feature is supported by everyone
-    bool Supported( int client );             // returns whether this feature is supported by a certain client ( resp. the server in client mode )
+    bool Supported() const;                   // returns whether this feature is supported by everyone
+    bool Supported( int client ) const;       // returns whether this feature is supported by a certain client ( resp. the server in client mode )
 private:
     int min_, max_;
 };
@@ -188,7 +208,7 @@ public:
     void			SetRate( unsigned short rate ){ rate_ = rate; }
     unsigned short	Rate()	{ return rate_; }
 
-    REAL			Control( Usage planned ){ return Usage_Planning == planned ? rateControlPlanned_ : rateControl_;}
+REAL			Control( Usage planned ){ return Usage_Planning == planned ? rateControlPlanned_ : rateControl_;}
     void			Use( Usage planned, REAL bandwidth );
 
     bool			CanSend(){ return rateControlPlanned_ > 0; }
@@ -217,6 +237,9 @@ public:
     void Timestep( REAL decay );           //!< lets all values decay, so they can be replaced by new ones
     void Add( REAL value, REAL weight=1 ); //!< adds a value to the average
     void Reset();                          //!< resets average to zero
+
+    std::istream & operator << ( std::istream & stream );       //!< read operator
+    std::ostream & operator >> ( std::ostream & stream ) const; //!< write operator
 private:
     REAL weight_;       //!< the total statistical weight
     REAL sum_;          //!< the total sum of value*weight
@@ -224,6 +247,12 @@ private:
     REAL weightSquared_;//!< the sum of all weights, squared
 };
 
+//! read operator for nAveragers
+std::istream & operator >> ( std::istream & stream, nAverager & averager );
+
+//! write operator for nAveragers
+std::ostream & operator << ( std::ostream & stream, nAverager const & averager );
+
 //! averager for pings, detects lag
 class nPingAverager
 {
@@ -271,7 +300,7 @@ struct nConnectionInfo     // everything that is needed to manage a connection
 
     nPingAverager          ping;
 
-    tCrypt*                crypt;
+    // tCrypt*                crypt;
 
     // rate control
     nBandwidthControl		bandwidthControl_;
@@ -286,7 +315,10 @@ struct nConnectionInfo     // everything that is needed to manage a connection
     tJUST_CONTROLLED_PTR< nMessage >          ackMess;
 
     // authentication
-    tString                userName;
+    // tString                userName;
+
+    // supported authentication methods of the client in a comma separated list
+    tString                 supportedAuthenticationMethods_;
 
     nConnectionInfo();
     ~nConnectionInfo();
@@ -458,10 +490,14 @@ public:
         return *this;
     }
 
+    // read a string without any kind of filtering
+    nMessage& ReadRaw(tString &s);
+
     nMessage& operator >> (tString &s);
     nMessage& operator >> (tColoredString &s);
     nMessage& operator << (const tString &s);
     nMessage& operator << (const tColoredString &s);
+    nMessage& operator << (const tOutput &o);
 
     template<class T> void BinWrite (const T &x){
         for(unsigned int i=0;i<sizeof(T)/2;i++)
@@ -550,6 +586,7 @@ protected:
     int id;
     tCONTROLLED_PTR(nMessage) message;  // the message
     int           receiver;      // the computer who should send the ack
+    REAL          timeout;       // the time in seconds between send attempts
     nTimeRolling  timeSendAgain; // for timeout
     nTimeRolling  timeFirstSent; // for ping calculation
     nTimeRolling  timeLastSent;  // for ping calculation
@@ -569,9 +606,18 @@ public:
 };
 
 
+//! pause a bit, abort pause on network activity
+void sn_Delay();
+
 // process the messages from all hosts and send acks
 void sn_Receive();
 
+// receive and process data from control socket (used on master server to ping servers)
+extern void sn_ReceiveFromControlSocket();
+
+// receive and discard data from control socket (used on regular servers to keep the pipe clean)
+extern void sn_DiscardFromControlSocket();
+
 // attempts to sync with server/all clients (<=> wait for all acks)
 // sync_netObjects: if set, network objects are synced as well
 // otherEnd: if set, the client instructs the server to send all packets and waits for completion.
@@ -580,7 +626,7 @@ void sn_Sync(REAL timeout,bool sync_sn_netObjects=false, bool otherEnd=true); //
 
 // causes the connected clients to print a message
 void sn_ConsoleOut(const tOutput &message,int client=-1);
-nMessage* sn_ConsoleOutMessage(const tOutput &message);
+nMessage* sn_ConsoleOutMessage( tString const & message );
 
 // causes the connected clients to print a message in the center of the screeen
 void sn_CenterMessage(const tOutput &message,int client=-1);
@@ -623,6 +669,12 @@ int sn_QueueLen(int user);
 
 void sn_Statistics();
 
+//! return the public IP address and port of this machine, or "*.*.*.*:*" if it is unknown..
+tString const & sn_GetMyAddress();
+
+//! checks wheter a given address is on the user's LAN (or on loopback).
+bool sn_IsLANAddress( tString const & address );
+
 // the SenderID of the currently handled message is stored here for reference
 class nCurrentSenderID
 {
@@ -658,6 +710,7 @@ private:
 class nMachine
 {
     friend class nMachineDecorator;
+    friend class nMachinePersistor;
 public:
     nMachine();          //!< constructor
     virtual ~nMachine(); //!< destructor
@@ -707,6 +760,28 @@ public:
     nMachine const & GetBanReason( tString & reason )const;//!< Gets the reason of the ban
     inline nMachineDecorator * GetDecorators( void ) const;	//!< Gets list of decorators
     inline nMachine const & GetDecorators( nMachineDecorator * & decorators ) const;	//!< Gets list of decorators
+
+    //! returns the next machine decorator of a given type
+    template< class T > static T * GetNextDecorator( nMachineDecorator * run )
+    {
+        while ( run )
+        {
+            T * ret = dynamic_cast< T * >( run );
+            if ( ret )
+            {
+                return ret;
+            }
+            run = run->Next();
+        }
+
+        return 0;
+    }
+
+    //! returns the first machine decorator of a given type
+    template< class T > T * GetDecorator()
+    {
+        return GetNextDecorator< T >( GetDecorators() );
+    }
 protected:
 private:
     inline nMachine & SetDecorators( nMachineDecorator * decorators );	//!< Sets list of decorators
@@ -720,6 +795,13 @@ public:
     ~nSocketResetInhibitor();
 };
 
+// on disconnection, this returns a server we should be redirected to (or NULL if we should not be redirected)
+std::auto_ptr< nServerInfoBase > sn_GetRedirectTo();
+
+// take a peek at the same info
+nServerInfoBase * sn_PeekRedirectTo();
+
+
 // *******************************************************************************************
 // *
 // *	GetWeight
