@@ -25,6 +25,9 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 */ // LoadLibrary
 
+// disable malloc replacement
+#define NO_MALLOC_REPLACEMENT
+
 #include "defs.h"
 
 #include <iostream>
@@ -33,11 +36,14 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "tMemManager.h"
 #include "tError.h"
 
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
 
 #ifdef WIN32
-#ifdef _MSC_VER
+#ifdef _MSC_VER 
 #include <windows.h>
-#include<winuser.h>
+#include <winuser.h>
 #ifdef _DEBUG
 #undef _DEBUG
 #endif
@@ -45,6 +51,13 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #pragma warning (disable : 4073)
 #pragma init_seg(lib)
 #endif
+#ifdef __MINGW32__
+#include <windows.h>
+#include <winuser.h>
+#ifdef _DEBUG
+#undef _DEBUG
+#endif
+#endif
 static CRITICAL_SECTION  mutex;
 #endif
 
@@ -73,6 +86,51 @@ static CRITICAL_SECTION  mutex;
 
 static bool reported=false;
 
+#ifdef HAVE_LIBZTHREAD
+#include <zthread/FastRecursiveMutex.h>
+
+static ZThread::FastRecursiveMutex st_mutex;
+#elif defined(HAVE_PTHREAD)
+#include "pthread-binding.h"
+static tPThreadRecursiveMutex st_mutex;
+#else
+class tMockMutex
+{
+public:
+    void acquire(){};
+    void release(){};
+};
+
+static tMockMutex st_mutex;
+#endif
+
+class tBottleNeck
+{
+public:
+    tBottleNeck()
+    {
+        st_mutex.acquire();
+    }
+
+    ~tBottleNeck()
+    {
+        st_mutex.release();
+    }
+};
+
+// replacement for wmemset function
+#ifndef HAVE_WMEMSET
+static inline wchar_t *wmemset(wchar_t *wcs, wchar_t wc, size_t n) throw()
+{
+    // fill memory
+    for( size_t i = 0; i < n; ++i )
+    {
+        wcs[i] = wc;
+    }
+
+    return wcs;
+}
+#endif
 
 void leak(){
     if (!reported)
@@ -85,7 +143,7 @@ void leak(){
         std::cerr << "\n\nMemory leak detected!\n\n";
 #endif
 #endif
-        tMemMan::Profile();
+        tMemManBase::Profile();
     }
 }
 
@@ -102,6 +160,38 @@ class memblock;
 #endif
 #define PAD 197
 
+// information about allocation in process
+struct tAllocationInfo
+{
+#ifdef LEAKFINDER
+    const char *filename;
+    const char *classname;
+    int checksum;
+    int line;
+    bool array;
+#endif
+
+    tAllocationInfo( bool 
+#ifdef LEAKFINDER
+                     array_ 
+#endif
+        )
+#ifdef LEAKFINDER
+    : filename( "XXX" ), classname( "XXX" ), checksum(-1), line(0), array( array_ )
+#endif
+    {
+    }
+};
+
+class tMemMan: public tMemManBase
+{
+public:
+    static void *Alloc(tAllocationInfo const & info, size_t s);
+    static void  Dispose(tAllocationInfo const & info, void *p);
+    static void  DisposeButKeep(tAllocationInfo const & info, void *p);
+};
+
+
 class tMemManager{
     friend class memblock;
 public:
@@ -109,13 +199,21 @@ public:
     tMemManager(int size);
     ~tMemManager();
 
-    void *      Alloc();
-    static void * AllocDefault(size_t size);
-    static void Dispose(void *p, bool keep=false);
+    void *      Alloc( tAllocationInfo const & info );
+    static void * AllocDefault(tAllocationInfo const & info, size_t size);
+    static void Dispose(tAllocationInfo const & info, void *p, bool keep=false);
     void        complete_Dispose(memblock *m);
     void        Check(); // check integrity
 
+    tMemManager( tMemManager const & other )
+    : size( other.size )
+    , blocksize( other.blocksize )
+    , semaphore( 1 )
+    {
+    }
 private:
+    tMemManager & operator =( tMemManager const & );
+
     int  Lower(int i){ // the element below i in the heap
         if(i%2==0)  // just to make sure what happens; you never know what 1/2 is..
             return i/2-1;
@@ -134,7 +232,7 @@ private:
     void CheckHeap(); // checks the heap structure
     //#endif
 
-static int  UpperL(int i){return 2*i+1;} // the elements left and
+    static int  UpperL(int i){return 2*i+1;} // the elements left and
     static int  UpperR(int i){return 2*i+2;} // right above i
 
     void Insert(memblock *b);  // starts to manage object e
@@ -159,11 +257,10 @@ static bool inited=false;
 #ifdef LEAKFINDER
 #include <fstream>
 #define MAXCHECKSUM 100001
-static int checksum=-1;
 static int counter[MAXCHECKSUM];
 static int leaks[MAXCHECKSUM];
 #ifdef PROFILER
-static const char *checksum_fileName[MAXCHECKSUM];
+static const char *checksum_filename[MAXCHECKSUM];
 static const char *checksum_classname[MAXCHECKSUM];
 static int         checksum_line[MAXCHECKSUM];
 static int         checksum_blocks[MAXCHECKSUM];
@@ -171,12 +268,8 @@ static int         checksum_bytes[MAXCHECKSUM];
 #endif
 
 
-static char *leakname="leak.log";
+static char const *leakname="leak.log";
 static bool checkleaks=true;
-static const char *fileName="XXX";
-static const char *classname="XXX";
-static int line;
-static bool array;
 #endif
 
 void begin_checkleaks(){
@@ -277,7 +370,7 @@ public: // everything is local to this file anyway...
     }
 
 
-    void *Alloc(){
+    void *Alloc( tAllocationInfo const & info ){
 #ifdef MEM_DEB_SMALL
         Check();
 #endif
@@ -312,25 +405,25 @@ public: // everything is local to this file anyway...
         //   tASSERT(_CrtCheckMemory());
 
 #ifdef LEAKFINDER
-        if (checksum >= 0){
-            tASSERT(checksum < MAXCHECKSUM);
+        if (info.checksum >= 0){
+            tASSERT(info.checksum < MAXCHECKSUM);
 
 #ifdef PROFILER
-            if (!counter[checksum]){
-                checksum_fileName [checksum] = fileName;
-                checksum_classname[checksum] = classname;
-                checksum_line     [checksum] = line;
+            if (!counter[info.checksum]){
+                checksum_filename [info.checksum] = info.filename;
+                checksum_classname[info.checksum] = info.classname;
+                checksum_line     [info.checksum] = info.line;
             }
-            checksum_blocks[checksum]++;
-            checksum_bytes[checksum]+=size;
+            checksum_blocks[info.checksum]++;
+            checksum_bytes[info.checksum]+=size;
             chunk(ret).realSize = size;
 #endif
-            chunk(ret).array=array;
-            chunk(ret).checksum=checksum;
-            chunk(ret).counter =++counter[checksum];
+            chunk(ret).array=info.array;
+            chunk(ret).checksum=info.checksum;
+            chunk(ret).counter =++counter[info.checksum];
 
             static int count =0;
-            if (checkleaks && counter[checksum] == leaks[checksum]){
+            if (checkleaks && counter[info.checksum] == leaks[info.checksum]){
                 count ++;
                 if (count <= 0)
                 {
@@ -355,8 +448,18 @@ public: // everything is local to this file anyway...
         }
 #endif
 
-#ifdef _DEBUG
-        memchr(data(ret), 0xfedabaac, size>>2);
+#ifdef LEAKFINDER
+#ifdef PROFILER
+        chunk(ret).realSize = size;
+#endif
+        chunk(ret).array=info.array;
+        chunk(ret).checksum=info.checksum;
+#endif
+
+#ifdef DEBUG
+        wmemset(static_cast< wchar_t * >( data(ret) ), wchar_t(0xfedabeef), size / sizeof( wchar_t ));
+#else
+        wmemset(static_cast< wchar_t * >( data(ret) ), 0, size / sizeof( wchar_t ));
 #endif
 
 #ifdef MEM_DEB_SMALL
@@ -369,7 +472,7 @@ public: // everything is local to this file anyway...
         return NULL;
     }
 
-    static memblock * Dispose(void *p, int &size,bool keep=false){
+    static memblock * Dispose(void *p, int &size, tAllocationInfo const & info, bool keep=false){
         chunkinfo &c=((chunkinfo *)p)[-1];
         size=c.size_in_dwords*4;
 
@@ -385,7 +488,7 @@ public: // everything is local to this file anyway...
 #endif
 
 #ifndef WIN32 // grr. on windows, this assertion fails on calls from STL.
-        tASSERT( c.array == array );
+        tASSERT( c.array == info.array );
 #endif
 
         if (size==0){
@@ -400,6 +503,11 @@ public: // everything is local to this file anyway...
         if (!c.occupied)
             return NULL;
 
+#ifdef DEBUG
+        wmemset(static_cast< wchar_t * >( p ), wchar_t(0xbadf00d), size / sizeof( wchar_t ));
+#else
+        wmemset(static_cast< wchar_t * >( p ), 0, size / sizeof( wchar_t ) );
+#endif
 
         memblock *myblock=(
                               (memblock *)(void *)
@@ -448,6 +556,7 @@ public: // everything is local to this file anyway...
     }
 
     void  Check(){
+#ifdef DEBUG
         tASSERT (size == myman->size);
 
         tASSERT (value >=0);
@@ -462,12 +571,13 @@ public: // everything is local to this file anyway...
                 tASSERT(a==b);
             }
         }
+#endif
     }
 
 #ifdef LEAKFINDER
     void dumpleaks(std::ostream &s){
         for (int i=myman->blocksize-1;i>=0;i--)
-            if (chunk(i).occupied){
+            if (chunk(i).occupied && chunk(i).checksum >= 0 ){
                 s << chunk(i).checksum << " " << chunk(i).counter << '\n';
                 leak();
             }
@@ -502,8 +612,13 @@ bool tMemManager::SwapIf(int i,int j){
 
 tMemManager::~tMemManager(){
 #ifdef LEAKFINDER
+    bool warn = true;
+#ifdef HAVE_LIBZTHREAD
+    warn = false;
+#endif
+
     if (inited){
-        // lsche das ding
+        // l???sche das ding
         std::ofstream lw(leakname);
         lw << "\n\n";
         tMemMan::Profile();
@@ -524,10 +639,10 @@ tMemManager::~tMemManager(){
             memblock::destroy(blocks(i));
         else{
 #ifdef LEAKFINDER
-            static bool warn = true;
             if ( warn )
             {
                 std::cout << "Memmanager warning: leaving block untouched.\n";
+                warn = false;
             }
             std::ofstream l(leakname,std::ios::app);
             blocks(i)->dumpleaks(l);
@@ -540,7 +655,11 @@ tMemManager::~tMemManager(){
             memblock::destroy(full_blocks(i));
         else{
 #ifdef LEAKFINDER
-            std::cout << "Memmanager warning: leaving block untouched.\n";
+            if ( warn )
+            {
+                std::cout << "Memmanager warning: leaving block untouched.\n";
+                warn = false;
+            }
             std::ofstream l(leakname,std::ios::app);
             full_blocks(i)->dumpleaks(l);
 #endif
@@ -555,6 +674,8 @@ tMemManager::tMemManager(int s,int bs):size(s),blocksize(s){
         blocksize=255;
     semaphore = 1;
 
+    tBottleNeck neck;
+
 #ifdef WIN32
     if (!inited)
         InitializeCriticalSection(&mutex);
@@ -564,6 +685,8 @@ tMemManager::tMemManager(int s,int bs):size(s),blocksize(s){
 }
 
 tMemManager::tMemManager(int s):size(s){//,blocks(1000),full_blocks(1000){
+    tBottleNeck neck;
+
     inited = false;
 
     blocks.SetLen(0);
@@ -576,6 +699,19 @@ tMemManager::tMemManager(int s):size(s){//,blocks(1000),full_blocks(1000){
 
 #ifdef LEAKFINDER
     if (size == 0){
+        for ( int i = MAXCHECKSUM-1; i>=0; --i )
+        {
+            leaks[i] = 0;
+            counter[i] = 0;
+#ifdef PROFILER
+            checksum_filename[i] = 0;
+            checksum_classname[i] = 0;
+            checksum_line[i] = 0;
+            checksum_blocks[i] = 0;
+            checksum_bytes[i] = 0;
+#endif
+        }
+
         std::ifstream l(leakname);
         while (l.good() && !l.eof()){
             int cs,ln;
@@ -586,7 +722,7 @@ tMemManager::tMemManager(int s):size(s){//,blocks(1000),full_blocks(1000){
         }
     }
     if (!inited){
-        // lsche das ding
+        // delete it
         std::ofstream lw(leakname);
         lw << "\n\n";
     }
@@ -730,11 +866,11 @@ memblock * tMemManager::Remove(int i){
     return NULL;
 }
 
-void * tMemManager::Alloc(){
+void * tMemManager::Alloc( tAllocationInfo const & info ){
     semaphore--;
     if (semaphore<0 || size == 0){
         semaphore++;
-        return AllocDefault(size);
+        return AllocDefault(info, size);
     }
 
 #ifdef WIN32
@@ -747,7 +883,7 @@ void * tMemManager::Alloc(){
 
     memblock *block=blocks(0);
     tASSERT (block->size == size);
-    void *mem = block->Alloc();
+    void *mem = block->Alloc( info );
     //tASSERT(_CrtCheckMemory());
     if (block->value == 0){
         Remove(block);
@@ -787,6 +923,7 @@ void tMemManager::complete_Dispose(memblock *block){
 }
 
 void tMemManager::Check(){
+#ifdef DEBUG_EXPENSIVE
     if ( !inited )
         return;
 
@@ -807,6 +944,7 @@ void tMemManager::Check(){
             tASSERT (full_blocks(i)->size == size);
         }
     }
+#endif
 }
 
 
@@ -927,15 +1065,16 @@ static tMemManager memman[MAX_SIZE+1]={
                                       };
 
 
-void tMemManager::Dispose(void *p,bool keep){
+void tMemManager::Dispose(tAllocationInfo const & info, void *p, bool keep){
     int size;
 
     if (!p)
         return;
 
-    memblock *block = memblock::Dispose(p,size,keep);
+    memblock *block = memblock::Dispose(p,size,info, keep);
 #ifndef DOUBLEFREEFINDER
     if (inited && block){
+        tBottleNeck neck;
         memman[size >> 2].complete_Dispose(block);
 #ifdef WIN32
         LeaveCriticalSection(&mutex);
@@ -946,20 +1085,23 @@ void tMemManager::Dispose(void *p,bool keep){
 #endif
 }
 
-void *tMemManager::AllocDefault(size_t s){
+void *tMemManager::AllocDefault(tAllocationInfo const & info, size_t s){
 #ifdef LEAKFINDER
     void *ret=malloc(s+sizeof(chunkinfo));//,classname,fileName,line);
-    ((chunkinfo *)ret)->checksum=checksum;
-    ((chunkinfo *)ret)->array=array;
+    ((chunkinfo *)ret)->checksum=info.checksum;
+    ((chunkinfo *)ret)->array=info.array;
 #ifdef PROFILER
     ((chunkinfo *)ret)->realSize=s;
-    if (!counter[checksum]){
-        checksum_fileName [checksum] = fileName;
-        checksum_classname[checksum] = classname;
-        checksum_line     [checksum] = line;
+    if ( info.checksum >= 0 )
+    {
+        if (!counter[info.checksum]){
+            checksum_filename [info.checksum] = info.filename;
+            checksum_classname[info.checksum] = info.classname;
+            checksum_line     [info.checksum] = info.line;
+        }
+        checksum_blocks[info.checksum]++;
+        checksum_bytes[info.checksum]+=s;
     }
-    checksum_blocks[checksum]++;
-    checksum_bytes[checksum]+=s;
 #endif
 #else
     void *ret=malloc(s+sizeof(chunkinfo));
@@ -970,7 +1112,7 @@ void *tMemManager::AllocDefault(size_t s){
     return ret;
 }
 
-void *tMemMan::Alloc(size_t s){
+void *tMemMan::Alloc(tAllocationInfo const & info, size_t s){
 #ifndef DONTUSEMEMMANAGER
 #ifdef MEM_DEB
 #ifdef WIN32
@@ -980,9 +1122,13 @@ void *tMemMan::Alloc(size_t s){
 #endif
     void *ret;
     if (inited && s < (MAX_SIZE << 2))
-        ret=memman[((s+3)>>2)].Alloc();
-    else{
-        ret=tMemManager::AllocDefault(s);
+    {
+        tBottleNeck neck;
+        ret=memman[((s+3)>>2)].Alloc( info );
+    }
+    else
+    {
+        ret=tMemManager::AllocDefault(info, s);
     }
 #ifdef MEM_DEB
     Check();
@@ -998,7 +1144,7 @@ void *tMemMan::Alloc(size_t s){
 
 }
 
-void tMemMan::DisposeButKeep(void *p){
+void tMemMan::DisposeButKeep( tAllocationInfo const & info, void *p){
 #ifndef DONTUSEMEMMANAGER
 #ifdef MEM_DEB
 #ifdef WIN32
@@ -1006,7 +1152,7 @@ void tMemMan::DisposeButKeep(void *p){
 #endif
     Check();
 #endif
-    tMemManager::Dispose(p,true);
+    tMemManager::Dispose(info, p, true);
 #ifdef MEM_DEB
     Check();
 #ifdef WIN32
@@ -1016,7 +1162,7 @@ void tMemMan::DisposeButKeep(void *p){
 #endif
 }
 
-void tMemMan::Dispose(void *p){
+void tMemMan::Dispose(tAllocationInfo const & info, void *p){
 #ifndef DONTUSEMEMMANAGER
 #ifdef MEM_DEB
 #ifdef WIN32
@@ -1024,7 +1170,7 @@ void tMemMan::Dispose(void *p){
 #endif
     Check();
 #endif
-    tMemManager::Dispose(p);
+    tMemManager::Dispose(info, p );
 #ifdef MEM_DEB
     Check();
 #ifdef WIN32
@@ -1037,9 +1183,10 @@ void tMemMan::Dispose(void *p){
 }
 
 
-void tMemMan::Check(){
+void tMemManBase::Check(){
     if (!inited)
         return;
+
 #ifdef WIN32
     EnterCriticalSection(&mutex);
 #endif
@@ -1123,139 +1270,135 @@ void * operator new(
 
 
 void* _cdecl operator new	(size_t size) THROW_BADALLOC{
+    tAllocationInfo info( false );
+
 #ifdef LEAKFINDER
-    checksum = size;
-    array = false;
+#ifndef HAVE_LIBZTHREAD
+    info.checksum = size;
 #endif
-    return tMemMan::Alloc(size);
-};
+#endif
+    return tMemMan::Alloc(info, size);
+}
 
 void  _cdecl operator delete(void *ptr) THROW_NOTHING{
-#ifdef LEAKFINDER
-    checksum = -1;
-    array = false;
-#endif
+    tAllocationInfo info( false );
+
     if (ptr)
-    tMemMan::Dispose(ptr);
-};
+        tMemMan::Dispose(info, ptr);
+}
 
 void  operator delete(void *ptr,bool keep) THROW_NOTHING{
-#ifdef LEAKFINDER
-    checksum = -1;
-    array = false;
-#endif
-    if (ptr){
-    if (keep)
-            tMemMan::DisposeButKeep(ptr);
-        else
-            tMemMan::Dispose(ptr);
+    tAllocationInfo info( false );
 
+    if (ptr){
+        if (keep)
+            tMemMan::DisposeButKeep(info, ptr);
+        else
+            tMemMan::Dispose(info, ptr);
+        
     }
-};
+}
 
 
 
 void* operator new	(size_t size,const char *classn,const char *file,int l) THROW_BADALLOC{
+    tAllocationInfo info( false );
 #ifdef LEAKFINDER
-    fileName=file;
-    classname=classn;
-    line=l;
-    array = false;
+    info.filename=file;
+    info.classname=classn;
+    info.line=l;
 
-    checksum =
+    info.checksum =
 #ifndef PROFILER 
         size +
 #endif 
-        line * 19671;
+        info.line * 19671;
 
     int c=1379;
     while (*file){
-    checksum = (checksum + (*file)*c) % MAXCHECKSUM;
-        c        = (c * 79              ) % MAXCHECKSUM;
+        info.checksum = (info.checksum + (*file)*c) % MAXCHECKSUM;
+        c             = (c * 79              ) % MAXCHECKSUM;
         file++;
     }
     while (*classn){
-    checksum = (checksum + (*classn)*c) % MAXCHECKSUM;
-        c        = (c * 79                ) % MAXCHECKSUM;
+        info.checksum = (info.checksum + (*classn)*c) % MAXCHECKSUM;
+        c             = (c * 79                ) % MAXCHECKSUM;
         classn++;
     }
 
 #ifdef PROFILER
-    while (checksum_fileName[checksum] && (checksum_fileName[checksum] != fileName || checksum_line[checksum] != line))
-    checksum = (checksum+1) % MAXCHECKSUM;
+    while (checksum_filename[info.checksum] && (checksum_filename[info.checksum] != info.filename || checksum_line[info.checksum] != info.line))
+    info.checksum = (info.checksum+1) % MAXCHECKSUM;
 #endif
 
 #endif
-    return tMemMan::Alloc(size);
+    return tMemMan::Alloc(info, size);
 }
 
 void  operator delete   (void *ptr,const char *classname,const char *file,int line)  THROW_NOTHING{
+    tAllocationInfo info( false );
+
+    if (ptr) tMemMan::Dispose(info, ptr);
+}
+
+void* operator new[]	(size_t size) THROW_BADALLOC{
+    tAllocationInfo info( true );
 #ifdef LEAKFINDER
-        checksum = 0;
-        array = false;
+#ifndef HAVE_LIBZTHREAD
+    info.checksum = size % MAXCHECKSUM;
 #endif
-        if (ptr) tMemMan::Dispose(ptr);
-    }
-    void* operator new[]	(size_t size) THROW_BADALLOC{
-#ifdef LEAKFINDER
-            checksum = size;
-            array = true;
 #endif
-            return tMemMan::Alloc(size);
-        };
+    return tMemMan::Alloc(info, size);
+}
 
 void  operator delete[](void *ptr) THROW_NOTHING {
-#ifdef LEAKFINDER
-    checksum = -1;
-    array = true;
-#endif
+    tAllocationInfo info( true );
+
     if (ptr)
-    tMemMan::Dispose(ptr);
-};
+        tMemMan::Dispose(info, ptr);
+}
 
 
 
 void* operator new[]	(size_t size,const char *classn,const char *file,int l)  THROW_BADALLOC{
+    tAllocationInfo info( true );
 #ifdef LEAKFINDER
-    fileName=file;
-    classname=classn;
-    line=l;
-    array = true;
+    info.filename=file;
+    info.classname=classn;
+    info.line=l;
 
-    checksum =
+    info.checksum =
 #ifndef PROFILER 
-        size +
+    size +
 #endif 
-        line * 19671;
-
+    info.line * 19671;
+    
     int c=1379;
     while (*file){
-    checksum = (checksum + (*file)*c) % MAXCHECKSUM;
-        c        = (c * 79              ) % MAXCHECKSUM;
+        info.checksum = (info.checksum + (*file)*c) % MAXCHECKSUM;
+        c             = (c * 79              ) % MAXCHECKSUM;
         file++;
     }
     while (*classn){
-    checksum = (checksum + (*classn)*c) % MAXCHECKSUM;
-        c        = (c * 79                ) % MAXCHECKSUM;
+        info.checksum = (info.checksum + (*classn)*c) % MAXCHECKSUM;
+        c             = (c * 79                ) % MAXCHECKSUM;
         classn++;
     }
-
+    
 #ifdef PROFILER
-    while (checksum_fileName[checksum] && (checksum_fileName[checksum] != fileName || checksum_line[checksum] != line))
-    checksum = (checksum+1) % MAXCHECKSUM;
+    while (checksum_filename[info.checksum] && (checksum_filename[info.checksum] != info.filename || checksum_line[info.checksum] != info.line))
+    info.checksum = (info.checksum+1) % MAXCHECKSUM;
 #endif
 
 #endif
-    return tMemMan::Alloc(size);
+    return tMemMan::Alloc(info, size);
 }
 
 void  operator delete[]   (void *ptr,const char *classname,const char *file,int line)   THROW_NOTHING{
-#ifdef LEAKFINDER
-        checksum = 0;
-        array = true;
-#endif
-        if (ptr) tMemMan::Dispose(ptr);
-    }
+    tAllocationInfo info( true );
+
+    if (ptr) tMemMan::Dispose(info, ptr);
+}
 
 #endif
 
@@ -1264,8 +1407,8 @@ void  operator delete[]   (void *ptr,const char *classname,const char *file,int
 
 #ifdef PROFILER
     static int compare(const void *a, const void *b){
-        int A = checksum_bytes[*((int *)a)];
-        int B = checksum_bytes[*((int *)b)];
+        int A = checksum_bytes[*((const int *)a)];
+        int B = checksum_bytes[*((const int *)b)];
 
         if (A<B)
             return -1;
@@ -1286,7 +1429,8 @@ void  operator delete[]   (void *ptr,const char *classname,const char *file,int
 #define snprintf _snprintf
 #endif
 
-void  tMemMan::Profile(){
+void  tMemManBase::Profile(){
+    tBottleNeck neck;
 #ifdef PROFILER
 
     int sort_checksum[MAXCHECKSUM];
@@ -1315,7 +1459,7 @@ void  tMemMan::Profile(){
     for (i=size-1; i>=0; i--)
     {
         int cs = sort_checksum[i];
-        const char *fn = checksum_fileName[cs];
+        const char *fn = checksum_filename[cs];
         const char *cn = checksum_classname[cs];
 
         if (!fn)
@@ -1362,5 +1506,70 @@ void  tMemMan::Profile(){
 #endif
 }
 
+#ifdef HAVE_LIBEFENCE
+class eFence
+{
+public:
+    eFence()
+    {
+        EF_newFrame();
+    }
+    ~eFence()
+    {
+        EF_delFrame();
+    }
+};
 
+static eFence fence;
+#endif
 
+#ifndef DONTUSEMEMMANAGER
+void * real_calloc( size_t nmemb, size_t size ) // call calloc
+{
+    return calloc( nmemb, size );
+}
+
+void * real_malloc( size_t size )                // call malloc
+{
+    return malloc( size );
+}
+
+void   real_free( void * ptr )                   // call free
+{
+    free( ptr );
+}
+
+void * real_realloc( void * ptr, size_t size )   // call realloc
+{
+    return realloc( ptr, size );
+}
+
+char * real_strdup( char const * ptr )   // call strdup
+{
+    return strdup( ptr );
+}
+
+void * real_mmove( void * ptr, size_t size )     // take memory allocated by real_malloc or a C library function and move to managed memory
+{
+    // create new memory, copy, free old memory
+    void * ret = tNEW(char)[size];
+    memcpy( ret, ptr, size );
+    real_free( ptr );
+
+    return ret;
+}
+
+char * real_strmove( char * ptr )                // take C string allocated by real_malloc or a C library function and move to managed memory
+{
+    if ( ptr )
+        return static_cast< char * >( real_mmove( ptr, strlen( ptr ) + 1 ) );
+    else
+        return 0;
+}
+
+#endif // DONTUSEMEMMANAGER
+
+char * tStrDup( char const * s )
+{
+    return real_strmove( strdup( s ) );
+}
