@@ -43,6 +43,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "tRandom.h"
 #include "tRecorder.h"
 #include <stdlib.h>
+#include <cstdlib>
+#include <memory>
 
 #define AI_REACTION          0 
 #define AI_EMERGENCY         1 
@@ -414,6 +416,7 @@ static bool CheckLoop(const gCycle *a, const gCycle *b,
 
                 // detect early loop
                 if (run == b)
+                {
                     if (bClosedIn)
                     {
                         winding = 0;
@@ -421,6 +424,7 @@ static bool CheckLoop(const gCycle *a, const gCycle *b,
                     }
                     else
                         bClosedIn = true;
+                }
             }
             else
             {
@@ -863,8 +867,8 @@ gCycleMemoryEntry* gCycleMemory::Latest (int side)  const
     for (int i=memory.Len()-1; i>=0; i--)
     {
         gCycleMemoryEntry* m = memory(i);
-        if ((!ret || (m->max[side].dist > ret->max[side].dist)
-                && bool( m->cycle ) && m->cycle->Alive() ))
+        if ((!ret || ( (m->max[side].dist > ret->max[side].dist)
+                       && bool( m->cycle ) && m->cycle->Alive() ) ) )
             ret = memory(i);
     }
 
@@ -878,8 +882,8 @@ gCycleMemoryEntry* gCycleMemory::Earliest (int side)  const
     for (int i=memory.Len()-1; i>=0; i--)
     {
         gCycleMemoryEntry* m = memory(i);
-        if ((!ret || (m->min[side].dist < ret->min[side].dist)
-                && bool( m->cycle ) && m->cycle->Alive()))
+        if ((!ret || ( (m->min[side].dist < ret->min[side].dist)
+                       && bool( m->cycle ) && m->cycle->Alive() ) ) )
             ret = memory(i);
     }
     return ret;
@@ -951,10 +955,14 @@ static void CycleBlocksWayHelper(const gCycle *a, const gCycle *b,
 // of a
 // bDir tells the direction the wall of b is going (-1: to the left, 1:...)
 // bDist is the distance of b's wall to its start.
-void gAIPlayer::CycleBlocksWay(const gCycle *a, const gCycle *b,
+void gAIPlayer::CycleBlocksWay(const gCycleMovement *aa, const gCycleMovement *bb,
                                int aDir, int bDir, REAL bDist, int winding)
 {
-    tASSERT(a && b);
+
+
+    tASSERT(aa && bb);
+    gCycle * a = dynamic_cast< gCycle * >( const_cast< gCycleMovement * > ( aa ) );
+    gCycle * b = dynamic_cast< gCycle * >( const_cast< gCycleMovement * > ( bb ) );
 
     REAL aDist = a->GetDistance();
     aDir = (aDir > 0 ? 1 : 0);
@@ -1007,12 +1015,12 @@ void gAIPlayer::CycleBlocksWay(const gCycle *a, const gCycle *b,
 }
 
 // called whenever a cylce blocks the rim wall.
-void gAIPlayer::CycleBlocksRim(const gCycle *a, int aDir)
+void gAIPlayer::CycleBlocksRim(const gCycleMovement *a, int aDir)
 {
 }
 
 // called whenever a hole is ripped in a's wall at distance aDist.
-void gAIPlayer::BreakWall(const gCycle *a, REAL aDist)
+void gAIPlayer::BreakWall(const gCycleMovement *a, REAL aDist)
 {}
 
 
@@ -1149,9 +1157,6 @@ gAIPlayer::gAIPlayer():
     pingCharity = 300;
 
     NewObject();
-
-    // AI players don't need to log in
-    Auth();
 }
 
 void gAIPlayer::ConfigureAIs()  // ai configuration menu
@@ -1229,7 +1234,7 @@ void gAIPlayer::SetNumberOfAIs(int num, int minPlayers, int iq, int tries)
         for (i = se_PlayerNetIDs.Len()-1; i>=0; i--)
         {
             ePlayerNetID *p = se_PlayerNetIDs(i);
-            if ( !p->IsSpectating() )
+            if ( p->CurrentTeam() )
                 ++pcount;
         }
 
@@ -1335,6 +1340,10 @@ void gAIPlayer::SetTraceSide(int side)
         lazySideChange = -10;
 }
 
+// flag set if pathfinding is enabled. It's an expensive opeeration, so we just turn it
+// off if the PC can't handle it. Doesn't doo much good, anyway.
+static bool sg_pathEnabled = true;
+
 // state change:
 void gAIPlayer::SwitchToState(gAI_STATE nextState, REAL minTime)
 {
@@ -1365,6 +1374,10 @@ void gAIPlayer::SwitchToState(gAI_STATE nextState, REAL minTime)
         break;
     case AI_PATH:
         nextAbility = character->properties[AI_STATE_PATH];
+        if (!sg_pathEnabled )
+        {
+            nextAbility = 0;
+        }
         break;
     case AI_SURVIVE:
         break;
@@ -1386,6 +1399,12 @@ void gAIPlayer::SwitchToState(gAI_STATE nextState, REAL minTime)
 // state update functions:
 void gAIPlayer::ThinkSurvive(  ThinkData & data )
 {
+    if (!character)
+    {
+        st_Breakpoint();
+        return;
+    }
+
     REAL random = 0;
     // do nothing much. Rely on the emergency program.
     /*
@@ -1493,14 +1512,14 @@ void gAIPlayer::ThinkTrace( ThinkData & data )
 
         if (a < b)
             a = b;
-        if ( a > 0 && a < nextTurn || !left.front.edge)
+        if ( ( a > 0 && a < nextTurn ) || !left.front.edge)
             nextTurn = a;
     }
 
     nextTurn/= Object()->Speed() * .98f;
 
     REAL delay = Delay() * 1.5f;
-    if ((!Object()->CanMakeTurn() || success) && nextTurn > delay)
+    if ((!Object()->CanMakeTurn(1) || !Object()->CanMakeTurn(-1) || success) && nextTurn > delay)
         nextTurn = delay;
 
     if (nextTurn > .3f)
@@ -1558,7 +1577,6 @@ void gAIPlayer::ThinkTrace( ThinkData & data )
     return;
 }
 
-
 void gAIPlayer::ThinkPath( ThinkData & data )
 {
     int lr = 0;
@@ -1583,6 +1601,14 @@ void gAIPlayer::ThinkPath( ThinkData & data )
 
     if ( nextStateChange < se_GameTime() )
     {
+        if( !sg_pathEnabled )
+        {
+            SwitchToState(AI_SURVIVE, 5);
+            EmergencySurvive( data );
+
+            return;
+        }
+
         gSensor p(Object(),Object()->Position(), tDir);
         p.detect(REAL(.9999999));
         if (p.hit >=  .9999999)  // free line of sight to victim. Switch to close combat.
@@ -1600,11 +1626,38 @@ void gAIPlayer::ThinkPath( ThinkData & data )
     if (lastPath < se_GameTime() - 10)
         if (target->CurrentFace())
         {
+            if( !sg_pathEnabled )
+            {
+                // yeah, apparently, we can't go on with this. Bail out.
+                SwitchToState(AI_SURVIVE, 5);
+                EmergencySurvive( data );
+                return;
+            }
+
             Object()->FindCurrentFace();
+            REAL before = tRealSysTimeFloat();
             eHalfEdge::FindPath(Object()->Position(), Object()->CurrentFace(),
                                 target->Position(), target->CurrentFace(),
                                 Object(),
                                 path);
+
+            // calculate (and archive) time used for pathfinding
+            REAL used = tRealSysTimeFloat() - before;
+            static char const * section = "PATH_TIME";
+            tRecorder::PlaybackStrict( section, used );
+            tRecorder::Record( section, used );
+            static REAL usedAverage = 0;
+            const REAL decay = .1;
+            usedAverage = (usedAverage+used*decay)/(1+decay);
+
+            // disable pathfinding if it just takes too long.
+            if ( used > .06 || usedAverage > .03 )
+            {
+#ifdef DEBUG
+                con << "Path finding is too expensive for this PC. Disabling it.\n";
+#endif
+                sg_pathEnabled = false;
+            }
             lastPath = se_GameTime();
         }
 
@@ -1716,7 +1769,7 @@ void gAIPlayer::ThinkCloseCombat( ThinkData & data )
         p.detect(REAL(1));
         if (p.hit <=  .999999)  // no free line of sight to victim. Switch to path mode.
         {
-            SwitchToState(AI_PATH, 5);
+            SwitchToState(sg_pathEnabled ? AI_PATH : AI_SURVIVE, 5);
             EmergencySurvive( data );
             return;
         }
@@ -1981,6 +2034,12 @@ public:
 // emergency functions:
 bool gAIPlayer::EmergencySurvive( ThinkData & data, int enemyevade, int preferedSide)
 {
+    if (!character)
+    {
+        st_Breakpoint();
+        return false;
+    }
+
     gAISensor const & front = data.front;
     gAISensor const & left = data.left;
     gAISensor const & right = data.right;
@@ -2001,7 +2060,8 @@ bool gAIPlayer::EmergencySurvive( ThinkData & data, int enemyevade, int prefered
         log->Print();
         //      st_Breakpoint();
     }
-    last = log->entries[log->current-1].turn;
+    if ( log->current > 0 )
+        last = log->entries[log->current-1].turn;
 #endif
 
     triesLeft = (triesLeft * character->properties[AI_EMERGENCY])/10;
@@ -2024,7 +2084,7 @@ bool gAIPlayer::EmergencySurvive( ThinkData & data, int enemyevade, int prefered
     REAL range = Object()->Speed() * delay;
 
     // nothing we can do if we cannot make a turn immediately
-    if (!Object()->CanMakeTurn())
+    if (!Object()->CanMakeTurn(1) || !Object()->CanMakeTurn(-1))
         return false;
 
     //  bool dontCheckForLoop[2] = { false, false };
@@ -2520,7 +2580,7 @@ void gAIPlayer::RightBeforeDeath(int triesLeft) // is called right before the ve
     if ( nCLIENT == sn_GetNetState() )
         return;
 
-    if ( !character )
+    if (!character)
     {
         st_Breakpoint();
         return;
@@ -2641,7 +2701,7 @@ static gAISensor * sg_GetSensor( int currentDirectionNumber, gCycle const & obje
     int turns = 1;
     grid.Turn( direction, turn );
     eCoord dir = grid.GetDirection( direction );
-    while ( turn * ( origDir * dir ) > .01 && currentDirectionNumber != direction )
+    while ( !ret || ( turn * ( origDir * dir ) > .01 && currentDirectionNumber != direction ) )
     {
         // cast rays
         gAISensor * sensor = tNEW(gAISensor)(&object,object.Position(),dir, side, range, corridor*.5f, turn );
@@ -2754,24 +2814,27 @@ REAL gAIPlayer::Think(){
 
     REAL ret = 1;
 
-    ThinkData data( front, *left, *right);
-    switch (state)
-    {
-    case AI_SURVIVE:
-        ThinkSurvive(data);
-        break;
-    case AI_PATH:
-        ThinkPath(data);
-        break;
-    case AI_TRACE:
-        ThinkTrace(data);
-        break;
-    case AI_CLOSECOMBAT:
-        ThinkCloseCombat(data);
-        break;
+    //not the best solution, but still better than segfault...
+    if(left.get() != 0 && right.get() != 0) {
+        ThinkData data( front, *left, *right);
+        switch (state)
+        {
+        case AI_SURVIVE:
+            ThinkSurvive(data);
+            break;
+        case AI_PATH:
+            ThinkPath(data);
+            break;
+        case AI_TRACE:
+            ThinkTrace(data);
+            break;
+        case AI_CLOSECOMBAT:
+            ThinkCloseCombat(data);
+            break;
+        }
+        ActOnData( data );
+        ret = data.thinkAgain;
     }
-    ActOnData( data );
-    ret = data.thinkAgain;
 
 #ifdef DEBUG_X
     {
@@ -2882,6 +2945,10 @@ void gAIPlayer::Timestep(REAL time){
         return;
     }
 
+    // don't think if the object is not up to date
+    if ( Object() && Object()->LastTime() < time - EPS )
+        return;
+
     REAL ts=time-lastTime;
     lastTime=time;
 
