@@ -29,6 +29,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "gGame.h"
 #include "gLogo.h"
 #include "gServerFavorites.h"
+#include "gFriends.h"
 
 #include "nServerInfo.h"
 #include "nNetwork.h"
@@ -39,6 +40,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "rRender.h"
 
 #include "uMenu.h"
+#include "uInputQueue.h"
 
 #include "tMemManager.h"
 #include "tSysTime.h"
@@ -51,13 +53,14 @@ int gServerBrowser::lowPort  = 4534;
 
 int gServerBrowser::highPort = 4540;
 static bool continuePoll = false;
-static int sg_simultaneous = 5;
+static int sg_simultaneous = 20;
 static tSettingItem< int > sg_simultaneousConf( "BROWSER_QUERIES_SIMULTANEOUS", sg_simultaneous );
 
 static tOutput *sg_StartHelpText = NULL;
 
 nServerInfo::QueryType sg_queryType = nServerInfo::QUERY_OPTOUT;
-static tSettingItem< int > sg_query_type( "BROWSER_QUERY_FILTER", reinterpret_cast< int & >( sg_queryType ) );
+tCONFIG_ENUM( nServerInfo::QueryType );
+static tSettingItem< nServerInfo::QueryType > sg_query_type( "BROWSER_QUERY_FILTER", sg_queryType );
 
 class gServerMenuItem;
 
@@ -66,12 +69,24 @@ class gServerInfo: public nServerInfo
 {
 public:
     gServerMenuItem *menuItem;
+	bool show; //for server browser hiding
 
-    gServerInfo():menuItem(NULL)
+    gServerInfo():menuItem(NULL), show(true)
     {
     };
 
     virtual ~gServerInfo();
+
+    // during browsing, the whole server list consists of gServerInfos
+    static gServerInfo * GetFirstServer()
+    {
+        return dynamic_cast< gServerInfo * >( nServerInfo::GetFirstServer() );
+    }
+
+    gServerInfo * Next()
+    {
+        return dynamic_cast< gServerInfo * >( nServerInfo::Next() );
+    }
 };
 
 nServerInfo* CreateGServer()
@@ -93,6 +108,8 @@ class gServerMenu: public uMenu
     int sortKey_;
 
 public:
+    virtual void OnRender();
+
     void Update(); // sort the server view by score
     gServerMenu(const char *title);
     ~gServerMenu();
@@ -100,7 +117,7 @@ public:
     virtual void HandleEvent( SDL_Event event );
 
     void Render(REAL y,
-                const tOutput &servername, const tOutput &score,
+                const tString &servername, const tOutput &score,
                 const tOutput &users     , const tOutput &ping);
 
     void Render(REAL y,
@@ -165,6 +182,21 @@ static bool sg_RequestLANcontinuously = false;
 
 void gServerBrowser::BrowseMaster()
 {
+    BrowseSpecialMaster(0,"");
+}
+
+// the currently active master
+static nServerInfoBase * sg_currentMaster = 0;
+nServerInfoBase * gServerBrowser::CurrentMaster()
+{
+    return sg_currentMaster;
+}
+
+
+void gServerBrowser::BrowseSpecialMaster( nServerInfoBase * master, char const * prefix )
+{
+    sg_currentMaster = master;
+
     sg_RequestLANcontinuously = false;
 
     sn_ServerInfoCreator *cback = nServerInfo::SetCreator(&CreateGServer);
@@ -183,7 +215,7 @@ void gServerBrowser::BrowseMaster()
     sr_textOut=true;
 
     nServerInfo::DeleteAll();
-    nServerInfo::GetFromMaster();
+    nServerInfo::GetFromMaster( master, prefix );
     nServerInfo::Save();
 
     //  gLogo::SetBig(true);
@@ -202,8 +234,9 @@ void gServerBrowser::BrowseMaster()
     sg_TalkToMaster = false;
 
     nServerInfo::SetCreator(cback);
-}
 
+    sg_currentMaster = master;
+}
 
 void gServerBrowser::BrowseLAN()
 {
@@ -260,6 +293,12 @@ void gServerBrowser::BrowseServers()
       ConnectToServer(nServerInfo::GetFirstServer());
     */
     browser.Update();
+
+    // eat excess input the user made while the list was fetched
+    SDL_Event ignore;
+    REAL time;
+    while(su_GetSDLInput(ignore, time)) ;
+
     browser.Enter();
 
     nServerInfo::GetFromLANContinuouslyStop();
@@ -282,15 +321,20 @@ void gServerMenu::HandleEvent( SDL_Event event )
         switch (event.key.keysym.sym)
         {
         case(SDLK_LEFT):
-                        sortKey_ = ( sortKey_ + nServerInfo::KEY_MAX-1 ) % nServerInfo::KEY_MAX;
+            sortKey_ = ( sortKey_ + nServerInfo::KEY_MAX-1 ) % nServerInfo::KEY_MAX;
             Update();
             return;
             break;
         case(SDLK_RIGHT):
-                        sortKey_ = ( sortKey_ + 1 ) % nServerInfo::KEY_MAX;
+            sortKey_ = ( sortKey_ + 1 ) % nServerInfo::KEY_MAX;
             Update();
             return;
             break;
+		case(SDLK_m):
+			FriendsToggle();
+            Update();
+			return;
+			break;
         default:
             break;
         }
@@ -300,35 +344,81 @@ void gServerMenu::HandleEvent( SDL_Event event )
     uMenu::HandleEvent( event );
 }
 
+void gServerMenu::OnRender()
+{
+    uMenu::OnRender();
+
+    // next time the server list is to be resorted
+    static double sg_serverMenuRefreshTimeout=-1E+32f;
+
+    if (sg_serverMenuRefreshTimeout < tSysTimeFloat())
+    {
+        Update();
+        sg_serverMenuRefreshTimeout = tSysTimeFloat()+2.0f;
+    }
+}
 
 void gServerMenu::Update()
 {
     // get currently selected server
     gServerMenuItem *item = NULL;
     if ( selected < items.Len() )
+    {
         item = dynamic_cast<gServerMenuItem*>(items(selected));
+    }
     gServerInfo* info = NULL;
     if ( item )
     {
         info = item->GetServer();
     }
 
+    // keep the cursor position relative to the top, if possible
+    int selectedFromTop = items.Len() - selected;
+
     ReverseItems();
 
     nServerInfo::CalcScoreAll();
     nServerInfo::Sort( nServerInfo::PrimaryKey( sortKey_ ) );
 
     int mi = 1;
-    nServerInfo *run = nServerInfo::GetFirstServer();
+    gServerInfo *run = gServerInfo::GetFirstServer();
+	bool oneFound = false; //so we can display all if none were found
     while (run)
     {
-        if (mi >= items.Len())
-            tNEW(gServerMenuItem)(this);
-
-        gServerMenuItem *item = dynamic_cast<gServerMenuItem*>(items(mi));
-        item->SetServer(run);
+		//check friend filter
+		if (getFriendsEnabled())
+		{
+			run->show = false;
+			int i;
+			tString userNames = run->UserNames();
+			tString* friends = getFriends();
+			for (i = MAX_FRIENDS; i>=0; i--)
+			{
+				if (run->Users() > 0 && friends[i].Len() > 1 && userNames.StrPos(friends[i]) >= 0)
+				{
+					oneFound = true;
+					run->show = true;
+				}
+			}
+		}
         run = run->Next();
-        mi ++;
+	}
+
+	run = gServerInfo::GetFirstServer();
+	{
+   		while (run)
+    	{
+			if (run->show || oneFound == false)
+			{
+	        	if (mi >= items.Len())
+    		        tNEW(gServerMenuItem)(this);
+
+    	    	gServerMenuItem *item = dynamic_cast<gServerMenuItem*>(items(mi));
+    	    	item->SetServer(run);
+	    	    mi++;
+			}
+        	run = run->Next();
+		}
     }
 
     if (items.Len() == 1)
@@ -342,8 +432,11 @@ void gServerMenu::Update()
 
     ReverseItems();
 
-    // set cursor to currently selected server
-    if ( info )
+    // keep the cursor position relative to the top, if possible ( calling function will handle the clamping )
+    selected = items.Len() - selectedFromTop;
+
+    // set cursor to currently selected server, if possible
+    if ( info && info->menuItem )
     {
         selected = info->menuItem->GetID();
     }
@@ -425,8 +518,13 @@ void gServerMenu::Render(REAL y,
         if (ping.Len() > 1)
         {
             text.SetPos(static_cast<int>(1.35/c.GetCWidth())  - tColoredString::RemoveColors( ping ).Len() - 1, true );
+            text << "0xRESETT";
             text << " " << ping;
         }
+        else
+        {
+            text << "0xRESETT";
+        }
 
         if (users.Len() > 1)
         {
@@ -445,7 +543,7 @@ void gServerMenu::Render(REAL y,
 }
 
 void gServerMenu::Render(REAL y,
-                         const tOutput &servername, const tOutput &score,
+                         const tString &servername, const tOutput &score,
                          const tOutput &users     , const tOutput &ping)
 {
     tColoredString highlight, normal;
@@ -488,6 +586,8 @@ void gServerMenu::Render(REAL y,
 #endif /* DEDICATED */
 static bool sg_filterServernameColorStrings = true;
 static tSettingItem< bool > removeServerNameColors("FILTER_COLOR_SERVER_NAMES", sg_filterServernameColorStrings);
+static bool sg_filterServernameDarkColorStrings = true;
+static tSettingItem< bool > removeServerNameDarkColors("FILTER_DARK_COLOR_SERVER_NAMES", sg_filterServernameDarkColorStrings);
 
 void gServerMenuItem::Render(REAL x,REAL y,REAL alpha, bool selected)
 {
@@ -558,7 +658,9 @@ void gServerMenuItem::Render(REAL x,REAL y,REAL alpha, bool selected)
         }
 
         if ( sg_filterServernameColorStrings )
-            name << tColoredString::RemoveColors( server->GetName() );
+            name << tColoredString::RemoveColors( server->GetName(), false );
+	else if ( sg_filterServernameDarkColorStrings )
+            name << tColoredString::RemoveColors( server->GetName(), true );
         else
         {
             name << server->GetName();
@@ -598,7 +700,9 @@ void gServerMenuItem::RenderBackground()
         else
             players << tOutput( "$network_master_players_empty" );
         players << "\n" << tColoredString::ColorString(1,1,1);
-        players << tOutput( "$network_master_serverinfo", server->Release(), server->Url(), server->Options() );
+        tColoredString uri;
+        uri << server->Url() << tColoredString::ColorString(1,1,1);
+        players << tOutput( "$network_master_serverinfo", server->Release(), uri, server->Options() );
     }
 #endif
 }
@@ -705,27 +809,25 @@ bool gServerMenuItem::Event( SDL_Event& event )
 void gBrowserMenuItem::RenderBackground()
 {
     sn_Receive();
+    sn_SendPlanned();
 
     menu->GenericBackground();
     if (continuePoll)
     {
         continuePoll = nServerInfo::DoQueryAll(sg_simultaneous);
         sn_Receive();
-    }
-
-    static double timeout=-1E+32f;
-
-    if (timeout < tSysTimeFloat())
-    {
-        (static_cast<gServerMenu*>(menu))->Update();
-        timeout = tSysTimeFloat()+2.0f;
+        sn_SendPlanned();
     }
 
 #ifndef DEDICATED
     rTextField::SetDefaultColor( tColor(.8,.3,.3,1) );
 
+	tString sn2 = tString(tOutput("$network_master_servername"));
+	if (getFriendsEnabled()) //display that friends filter is on
+		sn2 << " - " << tOutput("$friends_enable");
+
     static_cast<gServerMenu*>(menu)->Render(.62,
-                                            tOutput("$network_master_servername"),
+                                            sn2,
                                             tOutput("$network_master_score"),
                                             tOutput("$network_master_users"),
                                             tOutput("$network_master_ping"));
@@ -773,7 +875,7 @@ gServerInfo *gServerMenuItem::GetServer()
     return server;
 }
 
-static char *sg_HelpText = "$network_master_browserhelp";
+static char const * sg_HelpText = "$network_master_browserhelp";
 
 gServerMenuItem::gServerMenuItem(gServerMenu *men)
         :gBrowserMenuItem(men, sg_HelpText), server(NULL), lastPing_(-100), favorite_(false)
