@@ -67,13 +67,26 @@ protected:
     REAL YPos(int num);
 public:
     static bool          wrap;
-    static bool          quickexit;
+    
+    // different quick exit types
+    enum QuickExit
+    {
+        QuickExit_Off  = 0, // no quick exit, keep going
+        QuickExit_Game = 1, // quit to game selection menu
+        QuickExit_Total = 2 // quit the program
+    };
+
+    static QuickExit     quickexit;
     static bool          exitToMain;
 
     tOutput              title;
 
     FUNCPTR IdleFunc(){return idle;}
     static void SetIdle(FUNCPTR idle_func) {idle=idle_func;}
+
+    // poll input, return true if ESC was pressed
+    static bool IdleInput();
+
     void SetCenter(REAL c) {center=c;}
     void SetTop(REAL t) {menuTop=t;}
     void SetBot(REAL b) {menuBot=b;spaceBelow=1+menuBot;}
@@ -106,7 +119,7 @@ public:
     }
 
     // print a big message and a small interpretation
-    static void Message(const tOutput& message, const tOutput& interpretation, REAL timeout = -1);
+    static bool Message(const tOutput& message, const tOutput& interpretation, REAL timeout = -1);
 
     //! returns whether there is currently an active menu
     static bool MenuActive();
@@ -119,6 +132,9 @@ protected:
 
     //! marks the menu for exit
     virtual void OnExit();
+
+    //! called every frame before the menu is rendered
+    virtual void OnRender();
 };
 
 
@@ -134,7 +150,7 @@ protected:
     tOutput helpText;
 
     void DisplayText(REAL x,REAL y,const char *text,bool selected,
-                     REAL alpha=1,int center=0,int cursor=0,int cursorPos=0);
+                     REAL alpha=1,int center=0,int cursor=0,int cursorPos=0, rTextField::ColorMode colorMode = rTextField::COLOR_USE );
     void DisplayTextSpecial(REAL x,REAL y,const char *text,bool selected,
                             REAL alpha=1,int center=0);
 public:
@@ -355,8 +371,32 @@ public:
     virtual void Render(REAL x,REAL y,REAL alpha=1,bool selected=0);
 };
 
+// *****************************************
+//               Float Choose
+// *****************************************
 
+class uMenuItemReal:public uMenuItem{
+protected:
+    tOutput title;
+    REAL &target;
+    REAL Min,Max;
+    REAL Step;
+public:
+    /*
+      uMenuItemInt(uMenu *m,const char *tit,
+      const char *help,int &targ,
+      int mi,int ma,int step=1);
+    */
+    uMenuItemReal(uMenu *m,const tOutput &title,
+                 const tOutput &help,REAL &targ,
+                 REAL mi,REAL ma,REAL step=1);
 
+    ~uMenuItemReal(){};
+
+    virtual void LeftRight(int);
+
+    virtual void Render(REAL x,REAL y,REAL alpha=1,bool selected=0);
+};
 
 // *****************************************************
 //  String query
@@ -369,6 +409,9 @@ protected:
     tString *content;
     int      cursorPos;
     int		maxLength_;
+
+    // color mode used for rendering
+    rTextField::ColorMode colorMode_;
 public:
     uMenuItemString(uMenu *M,const tOutput& desc,
                     const tOutput& help,tString &c, int maxLength = 1024 );
@@ -379,6 +422,16 @@ public:
     virtual bool Event(SDL_Event &e);
 
     uMenu *MyMenu(){return menu;}
+
+    void SetColorMode( rTextField::ColorMode colorMode )
+    {
+        colorMode_ = colorMode;
+    }
+
+    rTextField::ColorMode GetColorMode() const
+    {
+        return colorMode_;
+    }
 };
 
 class uMenuItemStringWithHistory : protected uMenuItemString {
@@ -496,12 +549,12 @@ public:
 
     virtual ~uMenuItemFileSelection() {};
 
-    void SetDir( const char *dir ) { dir_ = dir; }; const
-    void SetFileSpec( const char *fileSpec ) { fileSpec_ = fileSpec; }; const
-    void SetFormatName( bool formatName ) { formatName_ = formatName; }; const
-    void SetGetFilesFlag( int getFilesFlag ) { getFilesFlag_ = getFilesFlag; } const
-    void SetDefaultFileName( const char *defaultFileName ) { defaultFileName_ = defaultFileName; }; const
-    void SetDefaultFilePath( const char *defaultFilePath ) { defaultFilePath_ = defaultFilePath; }; const
+    void SetDir( const char *dir ) { dir_ = dir; }
+    void SetFileSpec( const char *fileSpec ) { fileSpec_ = fileSpec; }
+    void SetFormatName( bool formatName ) { formatName_ = formatName; }
+    void SetGetFilesFlag( int getFilesFlag ) { getFilesFlag_ = getFilesFlag; }
+    void SetDefaultFileName( const char *defaultFileName ) { defaultFileName_ = defaultFileName; }
+    void SetDefaultFilePath( const char *defaultFilePath ) { defaultFilePath_ = defaultFilePath; }
 
     void SetParams( const char *dir, const char *fileSpec, int getFilesFlag,
                     bool formatName, const char *defaultFileName, const char *defaultFilePath )
